pub enum HandleError {
Comm(Errno),
NoFile,
NoAssociate(String),
Child,
Input,
Signal,
Io(Error),
Timeout,
User(Errno),
Errno(Errno),
}Expand description
Errors related to a ProcessHandle
Variants
Comm(Errno)
Errors related to communicating with the process, such as when waiting, killing, or sending a signal fails.
NoFile
Errors when a Handle’s descriptor functions are called, but the Spawner made no such descriptors.
NoAssociate(String)
Errors when no associate has the provided name.
Child
Errors when the Child fails; returned when the Handle’s readers get strange output from the child.
Input
Error when a Handle tries to write to a child standard input, but the child no longer exist.
Signal
The parent received a termination signal
Io(Error)
Error trying to write to standard input.
Timeout
Timeout error
User(Errno)
User switching errors.
Errno(Errno)
User switching errors.