pub enum StreamMode {
Pipe,
Share,
Log(Level),
Discard,
Fd(OwnedFd),
}Expand description
How to handle the standard input/out/error streams
Variants
Pipe
Collect the stream contents in a Stream object via a
pipe that can be retrieved in the spawn::Handle
Share STDIN/STDOUT/STDERR with the process, such that it can write to the parent. This is the default.
Log(Level)
Send the output to the system logger at the provided level. If the log level is below this, output is discarded.
Discard
Send output to /dev/null.
Fd(OwnedFd)
Send the output to the provided File Descriptor.